{-----------------------------------------------------------------------------
 Package:        MD - CP Gateway Service
 Developer(s):   Brian Juergensmeyer
 Date Created:   13-Oct-2006
 Site Name:      Hines OIFO
 Description:    This class is used by uCPNotifier.pas.  When a message is
                 received by TCPNotifier, it will parse this message into
                 either a TCPHL7Message or a TCPQueryMessage.  Note that the
                 TCPHL7Message will not contain the HL7, only the notification
                 that VistA has an incoming HL7 message ready for processing.
 History:
   $Header: /CP Modernization/CliO/source/CP Gateway/uCPMessage.pas 4     9/05/08 10:09p Zzzzzzpetitd $
   $History: uCPMessage.pas $
 * 
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 9/05/08    Time: 10:09p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Gateway work!
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 10/24/07   Time: 9:26a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzjuergb Date: 9/14/07    Time: 2:24p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Build 257
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzjuergb Date: 10/13/06   Time: 10:55a
 * Created in $/CP Modernization/CliO/source/CP Gateway
 * Initial checkin
-----------------------------------------------------------------------------}
unit uCPMessage;

interface

uses
  Classes, SysUtils, Contnrs, uCPParams, oGateway;

type
  ECPMessageError = class(Exception);

  TCPMessage = class(TPersistent)
  private
    FID: String;
    FMessage: TStringList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure AssignTo(Dest: TPersistent); override;
    function ToXML: string; virtual; abstract;
  published
    property ID: string read FID write FID;
    property Message: TStringList read FMessage write FMessage;
  end;

  TCPHL7Message = class(TCPMessage)
  private
  protected
  public
    function ToXML: string; override;
  published
  end;

  TCPQueryMessage = class(TCPMessage)
  private
    FParams: TCPParams;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    function ToXML: string; override;
  published
    property Params: TCPParams read FParams write FParams;
  end;

  function GenerateMessage(const VistAMessage: String; FLogger: ICPGatewayLogger): TCPMessage;

implementation

uses
  XMLDoc, XMLIntf, XMLDom, Windows, ActiveX, Variants;

const
  MESSAGE_ = 'message';
  TYPE_ = 'type';
  ID = 'id';
  TEXT = 'text';
  PARAMS = 'params';

  MSG_TYPE_COUNT = 2;
  CP_MESSAGE_TYPES: array[0..MSG_TYPE_COUNT - 1] of string =
    (
      'hl7',
      'query'
    );

function IndexOfMsgType(const MsgType: String): integer;
begin
  for Result := 0 to MSG_TYPE_COUNT - 1 do
  begin
    if CP_MESSAGE_TYPES[Result] = MsgType then
      Exit;
  end;
  result := -1;
end;

function GenerateMessage(const VistAMessage: String; FLogger: ICPGatewayLogger): TCPMessage;
var
  Doc: IXMLDocument;
  RootNode: IXMLNode;
  TypeNode: IXMLNode;
  IDNode: IXMLNode;
  TextNode: IXMLNode;
  ParamsNode: IXMLNode;
begin
  CoInitialize(nil);
  try
    try
      Doc := NewXMLDocument;
    except on E:Exception do
      FLogger.Log(Format('Unable to generate new XML document: %s', [E.Message]), llError);
    end;

    try
      FLogger.Log('Generating message from XML.', llDetail);
      try
        Doc.XML.Text := VistAMessage;
        Doc.Active := true;
      except on E:Exception do
        FLogger.Log(Format('Unable to generate message from XML: %s', [E.Message]), llError);
      end;

      RootNode := Doc.DocumentElement;
      if not Assigned(RootNode) then
        FLogger.Log('Unable to find root XML node for message.', llError);

      if RootNode.NodeName <> MESSAGE_ then
        FLogger.Log(Format('Message node named incorrectly.  Should be %s instead of %s.',
          ['message', RootNode.NodeName]), llError);

      TypeNode := RootNode.AttributeNodes.FindNode(TYPE_);
      if not assigned(TypeNode) then
        FLogger.Log('No message type was assigned in VistA XML message.', llError);


      IDNode := RootNode.ChildNodes.FindNode(ID);
      if not Assigned(IDNode) then
        FLogger.Log('No message ID available in message', llError);

      TextNode := RootNode.ChildNodes.FindNode(TEXT);

      if not Assigned(TextNode) then
        FLogger.Log('No message text available in message', llError);

      case IndexOfMsgType(TypeNode.NodeValue) of
      0: begin
            result := TCPHL7Message.Create;
            FLogger.Log('Generated HL7 message', llDetail);
         end;
      1: begin
           result := TCPQueryMessage.Create;
           FLogger.Log('Generated query message', llDetail);
         end;
      else
        begin
          result := nil;
          FLogger.Log(Format('Unknown message type: %s', [TypeNode.NodeValue]), llError);
          exit;
        end;
      end;

      result.FID := Trim(IDNode.NodeValue);
      FLogger.Log(Format('Generated message with id %s', [result.FID]), llDetail);

      if TextNode.NodeValue <> null then
      begin
        result.FMessage.Text := Trim(TextNode.NodeValue);
        FLogger.Log(Format('Generated message with text %s', [result.FMessage.text]), llDetail);
      end;


      if result is TCPQueryMessage then
      begin
        ParamsNode := RootNode.ChildNodes.FindNode(PARAMS);
        if Assigned(ParamsNode) then
          TCPQueryMessage(result).FParams.ParseFromXML(ParamsNode);
      end;

      FLogger.Log('Finished parsing XML document.', llDetail);
    finally
      Doc := nil;
    end;
  finally
    CoUninitialize;
  end;
end;

{ TCPMessage }

procedure TCPMessage.Assign(Source: TPersistent);
var
  SourceMessage: TCPMessage;
begin
  if Source is TCPMessage then
  begin
    SourceMessage := TCPMessage(Source);
    FID := SourceMessage.FID;
    FMessage.Assign(SourceMessage.FMessage);
  end
  else
    inherited;
end;

procedure TCPMessage.AssignTo(Dest: TPersistent);
var
  DestMessage: TCPMessage;
begin
  if Dest is TCPMessage then
  begin
    DestMessage := TCPMessage(Dest);
    DestMessage.FID := FID;
    DestMessage.FMessage.Assign(FMessage);
  end
  else
    inherited;
end;

constructor TCPMessage.Create;
begin
  FMessage := TStringList.Create;
end;

destructor TCPMessage.Destroy;
begin
  FreeAndNil(FMessage);
  inherited;
end;

{ TCPQueryMessage }

constructor TCPQueryMessage.Create;
begin
  FParams := TCPParams.Create(TCPParam);
end;

destructor TCPQueryMessage.Destroy;
begin
  FreeAndNil(FParams);
  inherited;
end;

function TCPQueryMessage.ToXML: string;
begin
  //
end;

{ TCPHL7Message }

function TCPHL7Message.ToXML: string;
begin
  //
end;

end.
