unit uXML;

interface

uses
  DateUtils,
  SysUtils,
  StrUtils,
  Variants,
  XMLDoc,
  XMLIntf;

function XMLValueAsStr(aNode: IXMLNode; aTag: string; aDefault: string = ''): string;
function XMLValueAsInt(aNode: IXMLNode; aTag: string): integer; overload;
function XMLValueAsInt(aNode: IXMLNode; aTag: string; aDefault: integer): integer; overload;
function XMLValueAsDouble(aNode: IXMLNode; aTag: string; aDefault: Double = 0.0): Double;
function XMLValueAsDateTime(aNode: IXMLNode; aTag: string): TDateTime;
function XMLComment(aNode: IXMLNode; aComment: string): integer;

function XMLAttributeAsStr(aNode: IXMLNode; aTag: string; aDefault: string = ''): string;
function XMLAttributeAsInt(aNode: IXMLNode; aTag: string; aDefault: integer = 0): integer;
function XMLAttributeAsDouble(aNode: IXMLNode; aTag: string; aDefault: Double = 0.0): Double;
function XMLAttributeAsDateTime(aNode: IXMLNode; aTag: string): TDateTime;

function XMLDocumentFirstRecord(aXMLDocument: IXMLDocument): IXMLNode;
function XMLDocumentNextRecord(aXMLNode: IXMLNode): IXMLNode;
function XMLDocumentRecordCount(aXMLDocument: IXMLDocument): integer;
function XMLStringToDateTime(sDateTime: string): TDateTime;
function DateTimeToXMLString(aDateTime: TDateTime): string;

function IsValidString(aValue: string; aMinLength, aMaxLength: integer; aInvalidCharacter: array of string): boolean;
function IsValidNumber(aValue: Double; aMin, aMax: Double; aDecimal: integer = 0): boolean;

function NewGUID(aStrip: Boolean = False): string;

function IsFileBoyStr(aItem, aValue: string; aMin, aMax: integer; var aMessage: string): boolean;
function IsFileBoyInt(aItem, aValue: string; aMin, aMax: integer; var aMessage: string): boolean;
function IsFileBoyNum(aItem, aValue: string; aMin, aMax, aDec: integer; var aMessage: string): boolean;

implementation

function XMLValueAsStr(aNode: IXMLNode; aTag: string; aDefault: string = ''): string;
begin
  try
    Result := VarToStrDef(aNode.ChildValues[aTag], aDefault);
  except
    Result := aDefault;
  end;
end;

function XMLValueAsInt(aNode: IXMLNode; aTag: string): integer;
begin
  Result := StrToIntDef(XMLValueAsStr(aNode, aTag), -1);
end;

function XMLValueAsInt(aNode: IXMLNode; aTag: string; aDefault: integer): integer;
begin
  Result := StrToIntDef(XMLValueAsStr(aNode, aTag), aDefault);
end;

function XMLValueAsDouble(aNode: IXMLNode; aTag: string; aDefault: Double = 0.0): Double;
begin
  Result := StrToFloatDef(XMLValueAsStr(aNode, aTag), aDefault);
end;

function XMLValueAsDateTime(aNode: IXMLNode; aTag: string): TDateTime;
begin
  try
    Result := XMLStringToDateTime(XMLValueAsStr(aNode, aTag));
  except
    Result := Now;
  end;
end;

function XMLAttributeAsStr(aNode: IXMLNode; aTag: string; aDefault: string = ''): string;
begin
  try
    Result := VarToStrDef(aNode.Attributes[aTag], aDefault);
  except
    Result := aDefault;
  end;
end;

function XMLAttributeAsInt(aNode: IXMLNode; aTag: string; aDefault: integer = 0): integer;
begin
  Result := StrToIntDef(XMLAttributeAsStr(aNode, aTag), aDefault);
end;

function XMLAttributeAsDouble(aNode: IXMLNode; aTag: string; aDefault: Double = 0.0): Double;
begin
  Result := StrToFloatDef(XMLAttributeAsStr(aNode, aTag), aDefault);
end;

function XMLAttributeAsDateTime(aNode: IXMLNode; aTag: string): TDateTime;
begin
  Result := XMLStringToDateTime(XMLAttributeAsStr(aNode, aTag));
end;

function XMLComment(aNode: IXMLNode; aComment: string): integer;
begin
  Result := aNode.ChildNodes.Add(aNode.OwnerDocument.CreateNode(aComment, ntComment));
end;

function DateTimeToXMLString(aDateTime: TDateTime): string;
begin
  Result := FormatDateTime('YYYY-MM-DD HH:MM:SS', aDateTime);
end;

function XMLStringToDateTime(sDateTime: string): TDateTime;
begin
  try
    Result := EncodeDateTime(
      StrToInt(Copy(sDateTime, 1, 4)),
      StrToInt(Copy(sDateTime, 6, 2)),
      StrToInt(Copy(sDateTime, 9, 2)),
      StrToInt(Copy(sDateTime, 12, 2)),
      StrToInt(Copy(sDateTime, 15, 2)),
      StrToInt(Copy(sDateTime, 18, 2)),
      0);
  except
    Result := 0;
  end;
end;

function XMLDocumentFirstRecord(aXMLDocument: IXMLDocument): IXMLNode;
begin
  aXMLDocument.Active := True;
  Result := aXMLDocument.DocumentElement.ChildNodes.FindNode('RECORD');
end;

function XMLDocumentNextRecord(aXMLNode: IXMLNode): IXMLNode;
begin
  Result := aXMLNode.NextSibling;
end;

function XMLDocumentRecordCount(aXMLDocument: IXMLDocument): integer;
var
  aNode: IXMLNode;
begin
  Result := 0;
  aNode := XMLDocumentFirstRecord(aXMLDocument);
  while Assigned(aNode) do
    begin
      inc(Result);
      aNode := XMLDocumentNextRecord(aNode);
    end;
end;

function IsValidString(aValue: string; aMinLength, aMaxLength: integer; aInvalidCharacter: array of string): boolean;
var
  i: integer;
begin
  if (Length(aValue) <= aMaxLength) and (Length(aValue) >= aMinLength) then
    begin
      for i := Low(aInvalidCharacter) to High(aInvalidCharacter) do
        if pos(aInvalidCharacter[i], aValue) > 0 then
          begin
            Result := False;
            Exit;
          end;
      Result := True;
    end
  else
    Result := False;
end;

function IsValidNumber(aValue: Double; aMin, aMax: Double; aDecimal: integer = 0): boolean;
begin
  Result := False;
end;

function NewGUID(aStrip: Boolean = False): string;
var
  aGUID: TGUID;
begin
  CreateGUID(aGUID);
  Result := GUIDToString(aGUID);
  if aStrip then
    begin
      Result := AnsiReplaceStr(Result, '{', '');
      Result := AnsiReplaceStr(Result, '}', '');
      Result := AnsiReplaceStr(Result, '-', '');
    end;
end;

{Fileman stuff - eventually this needs to be it's very own unit because fileboy is sooooooo special}

function FileBoyMsgInvalidCharacters(aItem, aChars: string): string;
begin
  Result := Format(
    'Value for ''%s'' cannot contain ''%s''.',
    [aItem, aChars]);
end;

function FileBoyMsgInvalidNumber(aItem: string; aMin, aMax, aDec: integer): string;
var
  aFmt: string;
begin
  aFmt := Format(
    'Value for ''%s'' must be between %0.%%df and %0.%%df with %d decimals are allowed.',
    [aItem, aDec, aDec, aDec]);
  Result := Format(aFmt, [aMin, aMax]);
end;

function FileBoyMsgInvalidLength(aItem: string; aMin, aMax: integer): string;
begin
  Result := Format(
    'Value for ''%s'' must be between %d and %d characters.',
    [aItem, aMin, aMax]);
end;

function IsFileBoyStr(aItem, aValue: string; aMin, aMax: integer; var aMessage: string): boolean;
begin
  Result := False;
  if Pos('^', aValue) > 0 then
    aMessage := FileBoyMsgInvalidCharacters(aItem, '^')
  else
    if Pos('@', aValue) > 0 then
      aMessage := FileBoyMsgInvalidCharacters(aItem, '@')
    else
      if (Length(aValue) < aMin) or (Length(aValue) > aMax) then
        aMessage := FileBoyMsgInvalidLength(aItem, aMin, aMax)
      else
        Result := True;
end;

function IsFileBoyInt(aItem, aValue: string; aMin, aMax: integer; var aMessage: string): boolean;
var
  aInt: integer;
begin
  Result := False;
  try
    aInt := StrToInt(aValue);
    if (aInt < aMin) or (aInt > aMax) then
      aMessage := FileBoyMsgInvalidNumber(aItem, aMin, aMax, 0)
    else
      Result := True;
  except
    aMessage := FileBoyMsgInvalidNumber(aItem, aMin, aMax, 0);
  end;
end;

function IsFileBoyNum(aItem, aValue: string; aMin, aMax, aDec: integer; var aMessage: string): boolean;
var
  aNum: Double;
begin
  Result := False;
  try
    aNum := StrToFloat(aValue);
    if (aNum < aMin) or (aNum > aMax) then
      aMessage := FileBoyMsgInvalidNumber(aItem, aMin, aMax, aDec)
    else
      Result := True;
  except
    aMessage := FileBoyMsgInvalidNumber(aItem, aMin, aMax, aDec);
  end;
end;

end.

