unit uServerList;

interface

uses
  Classes, SysUtils;

type
  TServers = class;

  TServerItem = class(TCollectionItem)
  private
    FName: string;
    FIP: string;
    FPort: Integer;
  public
    constructor Create(Servers: TServers);
    destructor Destroy; override;
    procedure Assign(Source:TPersistent); override;
    property Name: string read FName write FName;
    property IP: string read FIP write FIP;
    property Port: Integer read FPort write FPort;
  end;

  TServers = class(TCollection)
  protected
    function GetItem(Index: Integer): TServerItem;
    procedure SetItem(Index: Integer; Item: TServerItem);
  public
    constructor Create;
    function Add: TServerItem;
    procedure Assign(Source: TPersistent); override;
    function FindItemID(ID: Integer): TServerItem;
    function Insert(Index: Integer): TServerItem;

    procedure Load(const AFilename: string);
    procedure Save(const AFilename: string);

    property Items[Index: Integer]: TServerItem read GetItem write SetItem; default;
  end;

implementation

uses
  XMLIntf, XMLDoc;

const
  _SERVERS = 'SERVERS';
  _SERVER = 'SERVER';
  _IP_ADDR = 'IP_ADDRESS';
  _PORT = 'PORT';
  _NAME = 'NAME';

{ TServerItem }

procedure TServerItem.Assign(Source: TPersistent);
var
  Item: TServerItem;
begin
  if Source is TServerItem then
  begin
    Item := Source as TServerItem;
    Name := Item.Name;
    IP := Item.IP;
    Port := Item.Port;
  end
  else
    inherited;
end;

constructor TServerItem.Create(Servers: TServers);
begin
  inherited Create(Servers);
end;

destructor TServerItem.Destroy;
begin
  inherited;
end;

{ TServers }

function TServers.Add: TServerItem;
begin
  result := TServerItem(inherited Add);
end;

procedure TServers.Assign(Source: TPersistent);
var
  Enum: TCollectionEnumerator;
  Item: TServerItem;
begin
  if Source is TServers then
  begin
    Clear;

    Enum := TCollection(Source).GetEnumerator;

    while Enum.MoveNext do
    begin
      Item := Add;
      Item.Assign(Enum.GetCurrent);
    end;
  end
  else
    inherited;
end;

constructor TServers.Create;
begin
  inherited Create(TServerItem);
end;

function TServers.FindItemID(ID: Integer): TServerItem;
begin
  result := TServerItem(Inherited FindItemID(ID));
end;

function TServers.GetItem(Index: Integer): TServerItem;
begin
  result := TServerItem(Inherited GetItem(Index));
end;

function TServers.Insert(Index: Integer): TServerItem;
begin
  result := TServerItem(inherited Insert(Index));
end;

procedure TServers.Load(const AFilename: string);
var
  Doc: IXMLDocument;
  Servers: IXMLNode;
  Server: IXMLNode;
  IPNode: IXMLNode;
  PortNode: IXMLNode;
  I: Integer;
  Item: TServerItem;
begin
  Doc := NewXMLDocument;
  Doc.LoadFromFile(AFileName);

  Servers := Doc.ChildNodes.FindNode(_SERVERS);
  for I := 0 to Servers.ChildNodes.Count - 1 do
  begin
    Server := Servers.ChildNodes[I];
    IPNode := Server.ChildNodes.FindNode(_IP_ADDR);
    PortNode := Server.ChildNodes.FindNode(_PORT);

    Item := Add;
    Item.FName := Server.Attributes[_NAME];
    Item.FIP := IPNode.NodeValue;
    Item.FPort := PortNode.NodeValue;
  end;
end;

procedure TServers.Save(const AFilename: string);
var
  Doc: IXMLDocument;
  Servers: IXMLNode;
  Server: IXMLNode;
  IPNode: IXMLNode;
  PortNode: IXMLNode;
  I: Integer;
  Item: TServerItem;
begin
  Doc := NewXMLDocument;

  Servers := Doc.AddChild(_SERVERS);
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    Server := Servers.AddChild(_SERVER);
    Server.Attributes[_NAME] := Item.FName;
    IPNode := Server.AddChild(_IP_ADDR);
    PortNode := Server.AddChild(_PORT);
    IPNode.NodeValue := Item.FIP;
    PortNode.NodeValue := Item.FPort;
  end;
  Doc.Options := [doNodeAutoCreate, doNodeAutoIndent];
  Doc.SaveToFile(AFilename);
end;

procedure TServers.SetItem(Index: Integer; Item: TServerItem);
begin
  inherited SetItem(Index, Item);
end;

end.
